searchNodes=[{"doc":"OneAndDone makes it easy to introduce idempotency in any Elixir application. Its only dependency is a cache to store requests in (Nebulex works great). Usage is framework-dependent. Currently, OneAndDone supports the following frameworks: OneAndDone.Plug Other frameworks are welcome too! For details on integrating OneAndDone with your framework, look at the docs for the module you would use in your framework.","ref":"OneAndDone.html","title":"OneAndDone","type":"module"},{"doc":"Defines the most basic cache interface. This module is used as a reference for Cache implementations. Although not used by OneAndDone, Cache implementations should be compliant with this module. This module is compliant with Nebulex.Cache . If you use Nebulex, you are already compliant with this module.","ref":"OneAndDone.Cache.html","title":"OneAndDone.Cache","type":"behaviour"},{"doc":"Retreive a value from the cache.","ref":"OneAndDone.Cache.html#c:get/1","title":"OneAndDone.Cache.get/1","type":"callback"},{"doc":"Store a value in the cache under the given key. Opts must include a TTL, given in milliseconds.","ref":"OneAndDone.Cache.html#c:put/3","title":"OneAndDone.Cache.put/3","type":"callback"},{"doc":"Easy to use plug for idempoent requests. Getting started Add :one_and_done to your list of dependencies in mix.exs : def deps do [ { :one_and_done , &quot;~&gt; 0.1.1&quot; } ] end Add the plug to your router: pipeline :api do plug OneAndDone.Plug , # Required: must conform to OneAndDone.Cache (Nebulex.Cache works fine) cache : MyApp.Cache , # Optional: How long to keep entries, defaults to 86_400 (24 hours) ttl : 86_400 , # Optional: Which methods to cache, defaults to [&quot;POST&quot;, &quot;PUT&quot;] # Used by the default idempotency_key_fn to quickly determine if the request # can be cached. If you override idempotency_key_fn, consider checking the # request method in your implementation for better performance. # `supported_methods` is available in the opts passed to the idempotency_key_fn. supported_methods : [ &quot;POST&quot; , &quot;PUT&quot; ] , # Optional: Function reference to generate the idempotency key for a given request. # By default, uses the value of the &quot;Idempotency-Key&quot; header. # Must return a binary or nil. If nil is returned, the request will not be cached. # Default function implementation: # # fn conn, opts -&gt; # Opts is the same as the opts passed to the plug # if Enum.any?(opts.supported_methods, &amp;(&amp;1 == conn.method)) do # conn # |&gt; Plug.Conn.get_req_header(&quot;idempotency-key&quot;) # Request headers are always downcased # |&gt; List.first() # else # nil # end # end idempotency_key_fn : &amp; OneAndDone.Plug . idempotency_key_from_conn / 2 , # Optional: Function reference to generate the cache key for a given request. # Given the conn &amp; idempotency key (returned from idempotency_key_fn), this function # should return a term that will be used as the cache key. # By default, it returns a tuple of the module name and the idempotency key. # Default function implementation: fn _conn, idempotency_key -&gt; {__MODULE__, idempotency_key} cache_key_fn : &amp; OneAndDone.Plug . build_cache_key / 2 end That's it! POST and PUT requests will now be cached by default for 24 hours. Telemetry To monitor the performance of the OneAndDone plug, you can hook into OneAndDone.Telemetry . For a complete list of events, see OneAndDone.Telemetry.events/0 . Example # In your application.ex # ... :telemetry . attach_many ( &quot;one-and-done&quot; , OneAndDone.Telemetry . events ( ) , &amp; MyApp.Telemetry . handle_event / 4 , nil ) # ... # In your telemetry module: defmodule MyApp.Telemetry do require Logger def handle_event ( [ :one_and_done , :request , :stop ] , measurements , _metadata , _config ) do duration = System . convert_time_unit ( measurements . duration , :native , :millisecond ) Logger . info ( &quot;Running one_and_done took \#{ duration } ms&quot; ) :ok end # Catch-all for unhandled events def handle_event ( _ , _ , _ , _ ) do :ok end end","ref":"OneAndDone.Plug.html","title":"OneAndDone.Plug","type":"module"},{"doc":"A basic module for capturing the essence of a response. Response structs are stored in the cache so that idempotent requests can be quickly returned. See OneAndDone.Response.Parser for turning an inbound connection (e.g. a Plug.Conn) into a OneAndDone.Response .","ref":"OneAndDone.Response.html","title":"OneAndDone.Response","type":"module"},{"doc":"See OneAndDone.Response.Parser.build_response/1 .","ref":"OneAndDone.Response.html#build_response/1","title":"OneAndDone.Response.build_response/1","type":"function"},{"doc":"","ref":"OneAndDone.Response.html#t:t/0","title":"OneAndDone.Response.t/0","type":"type"},{"doc":"Protocol for turning an inbound connection (e.g. a Plug.Conn) into a OneAndDone.Response.","ref":"OneAndDone.Response.Parser.html","title":"OneAndDone.Response.Parser","type":"protocol"},{"doc":"","ref":"OneAndDone.Response.Parser.html#build_response/1","title":"OneAndDone.Response.Parser.build_response/1","type":"function"},{"doc":"","ref":"OneAndDone.Response.Parser.html#t:t/0","title":"OneAndDone.Response.Parser.t/0","type":"type"},{"doc":"Telemetry integration to track how long it takes to process a request. OneAndDone emits the following metrics: Metric Description Measurements Metadata [:one_and_done, :request, :start] When we begin processing a request. conn , opts [:one_and_done, :request, :stop] When we finish processing a request, including the duration in native units. duration conn , opts [:one_and_done, :request, :exception] When we finish processing a request, if an exception was raised. Includes the duration in native units. duration , exception conn , opts [:one_and_done, :request, :cache_hit] Given an idempotency key, we found a cached response. idempotency_key conn , response [:one_and_done, :request, :cache_miss] Given an idempotency key, we didn't find a cached response. idempotency_key conn [:one_and_done, :request, :idempotency_key_not_set] The request doesn't have an idempotency key and will not be processed further by OneAndDone. conn [:one_and_done, :request, :cache_get, :start] When we begin checking the cache for a request. conn , idempotency_key [:one_and_done, :request, :cache_get, :stop] When we finish checking the cache for a request, including the duration in native units. duration conn , idempotency_key [:one_and_done, :request, :cache_get, :exception] When we finish checking the cache for a request, if an exception was raised. Includes the duration in native units. duration , exception conn , idempotency_key [:one_and_done, :request, :cache_put, :start] When we begin serializing and putting a response into the cache. conn , idempotency_key [:one_and_done, :request, :cache_put, :stop] When we finish serializing and putting a response into the cache, including the duration in native units. duration conn , idempotency_key [:one_and_done, :request, :cache_put, :exception] When we finish serializing and putting a response into the cache, if an exception was raised. Includes the duration in native units. duration , exception conn , idempotency_key The duration is emitted in native units. To convert to milliseconds, use System.convert_time_unit(duration, :native, :millisecond) .","ref":"OneAndDone.Telemetry.html","title":"OneAndDone.Telemetry","type":"module"},{"doc":"Emit a telemetry event.","ref":"OneAndDone.Telemetry.html#event/3","title":"OneAndDone.Telemetry.event/3","type":"function"},{"doc":"Return the list of events emitted by this module.","ref":"OneAndDone.Telemetry.html#events/0","title":"OneAndDone.Telemetry.events/0","type":"function"},{"doc":"Measure the duration of a function call.","ref":"OneAndDone.Telemetry.html#span/3","title":"OneAndDone.Telemetry.span/3","type":"function"},{"doc":"Additional metadata to include at the end of a span.","ref":"OneAndDone.Telemetry.SpanResult.html","title":"OneAndDone.Telemetry.SpanResult","type":"module"},{"doc":"Create a new SpanResult struct.","ref":"OneAndDone.Telemetry.SpanResult.html#new/1","title":"OneAndDone.Telemetry.SpanResult.new/1","type":"function"},{"doc":"","ref":"OneAndDone.Telemetry.SpanResult.html#t:t/0","title":"OneAndDone.Telemetry.SpanResult.t/0","type":"type"},{"doc":"Raised when a cache is not configured. Check the docs for the OneAndDone module you are using (e.g. OneAndDone.Plug ) for details on how to configure a cache.","ref":"OneAndDone.Errors.CacheMissingError.html","title":"OneAndDone.Errors.CacheMissingError","type":"exception"},{"doc":"One and Done is the easiest way to make HTTP requests idempotent in Elixir applications. One and Done supports the following frameworks: Plug (including Phoenix )","ref":"readme.html","title":"One and Done","type":"extras"},{"doc":"One and Done depends on having a pre-existing cache like Nebulex . This guide assumes Nebulex is already configured under MyApp.Cache . Add one_and_done to your mix.exs dependencies: def deps do [ { :one_and_done , &quot;~&gt; 0.1.1&quot; } ] end Add OneAndDone to your Plug pipeline: defmodule MyAppWeb.Router do use MyAppWeb , :router pipeline :api do # Configuration options for OneAndDone are in the docs plug OneAndDone.Plug , cache : MyApp.Cache end # By default, all POST and PUT requests piped through :api # that have an Idempotency-Key header set will be cached for 24 hours. scope &quot;/api&quot; , MyAppWeb do pipe_through :api resources &quot;/users&quot; , UserController end end Make your requests idempotent by adding the Idempotency-Key header: curl -X POST \\ http://localhost:4000/api/users \\ -H &#39;Content-Type: application/json&#39; \\ -H &#39;Idempotency-Key: 123&#39; \\ -d &#39;{ &quot;email&quot;: &quot;hello@example.com&quot;, &quot;password&quot;: &quot;password&quot; }&#39; Repeat the request with the same Idempotency-Key header and you will get the same response without the request being processed again.","ref":"readme.html#usage","title":"One and Done - Usage","type":"extras"}]